﻿using FL.Battle.Buffs;
using System.Collections.Generic;
using XGame.Framework;
using XGame.Framework.Interfaces;

namespace FL.Map.TestMainMap
{
    public partial class EntitiesComponent: ILateUpdate
    {
        private HashSet<long> _buffOwnerIds;
        private bool _isElementBuffDirty;
        private BuffPreviewDto _buffPreview;
        void ILateUpdate.LateUpdate(int millisecond)
        {
            Context.VM.UpdateMapMoveTime(millisecond);
            Context.VM.Camera.OnLateUpdate(millisecond);
            RefreshElementBuffs();
        }
        #region 元素buff
        private void RefreshElementBuffs()
        {
            if (!_isElementBuffDirty) return;
            _isElementBuffDirty = false;
            if (_buffOwnerIds == null || _buffOwnerIds.Count == 0) return;
            var buffOwnerIds = ListPool.Acquire<long>();
            buffOwnerIds.AddRange(_buffOwnerIds);
            var elementBuffs = ListPool.Acquire<IBuff>();
            foreach (var ownerId in buffOwnerIds)
            {
                if (_entityViewsMap.TryGetValue(ownerId, out var view) && 
                    view is MonsterEntityView monster && 
                    monster.Buffs.GetElementBuffs(ref elementBuffs))
                {
                    continue;
                }
                //非怪物 || 没有满足要求的元素buff
                _buffOwnerIds.Remove(ownerId);
            }
            elementBuffs.Sort((a, b) =>
            { // 层数高的排前面
                var orderA = a.Layers * 1000 / a.LayerLimit;
                var orderB = b.Layers * 1000 / b.LayerLimit;
                return orderB - orderA;
            });
            var isMatch = false;
            foreach (var buff in elementBuffs)
            {
                foreach (var partner in _partnerMap.Values)
                {
                    if (MatchTargetBuff(partner, buff))
                    {
                        isMatch = true;
                        break;
                    }
                }
                if (isMatch)
                {
                    break;
                }
            }
            ListPool.Recycle(buffOwnerIds);
            ListPool.Recycle(elementBuffs);
            if (!isMatch)
            { // 没有匹配的项
                ResetBuffPreview();
            }
        }
        private bool MatchTargetBuff(PartnerEntityView partner, IBuff buff)
        {
            if (partner.Epigraphics.IsTargetBuff(buff.TableId, out var epigrapId))
            {
                NotifyBuffPreview(buff, partner.Entity.EntityId, false, epigrapId);
                return true;
            }
            if (partner.Skill.FindSkillByElementType(buff.ElementType, out var skillId))
            {
                NotifyBuffPreview(buff, partner.Entity.EntityId, true, skillId);
                return true;
            }
            return false;
        } 
        private void NotifyBuffPreview(IBuff buff, long partnerId, bool isSkill, int skillId)
        {
            _buffPreview = new BuffPreviewDto()
            {
                ownerId = buff.OwnerId,
                buffTableId = buff.TableId,
                partnerId = partnerId,
                isSkill = isSkill,
                skillId = skillId,
                layer = buff.Layers,
                layerLimit = buff.LayerLimit,
            };
            EventSingle.Instance.Notify(EventDefine.GameMainMapBuffPreview, _buffPreview);
        }
        private void ResetBuffPreview()
        {
            _buffPreview = default;
            EventSingle.Instance.Notify(EventDefine.GameMainMapBuffPreview, _buffPreview);
        }
        private void OnAddBuff(int eventId, object args)
        {
            var dto = (BuffEventDto)args;
            var buffOwners = _buffOwnerIds ??= new HashSet<long>();
            buffOwners.Add(dto.ownerId);
            _isElementBuffDirty = true;
        }
        private void OnRemoveBuff(int eventId, object args)
        {
            var dto = (BuffEventDto)args;
            if (_buffPreview.ownerId == dto.ownerId && _buffPreview.buffTableId == dto.buffTableId)
            {
               ResetBuffPreview();
               _isElementBuffDirty = true;
            }
        }
        private void RemoveElementBuff(long ownerId)
        {
            if (_buffPreview.ownerId == ownerId)
            {
                ResetBuffPreview();
            }
            if (_buffOwnerIds?.Remove(ownerId) ?? false)
            {
                _isElementBuffDirty = true;
            }
        }
        private void ClearElementBuffs()
        {
            _buffOwnerIds?.Clear();
            _isElementBuffDirty = false;
            _buffPreview = default;
        }
        #endregion
    }
}
